@@ -30,6 +30,7 @@
 #include "mapserver.h"
 #include "maptime.h"
 #include "maptemplate.h"
+#include "mapows.h"
 
 #if defined(USE_LIBXML2)
 #include "maplibxml2.h"
@@ -37,6 +38,7 @@
 #include "cpl_minixml.h"
 #include "cpl_error.h"
 #endif
+#include "mapowscommon.h"
 
 #include <ctype.h> /* isalnum() */
 #include <stdarg.h>
@@ -79,6 +81,17 @@ static void msOWSClearRequestObj(owsRequestObj *ows_request)
   }
 }
 
+#if defined(USE_LIBXML2) && LIBXML_VERSION < 20900
+static int bExternalEntityAsked = FALSE;
+static xmlParserInputPtr  dummyEntityLoader(const char * URL, 
+                                           const char * ID, 
+                                           xmlParserCtxtPtr context )
+{
+    bExternalEntityAsked = TRUE;
+    return NULL;
+}
+#endif
+
 /*
 ** msOWSPreParseRequest() parses a cgiRequestObj either with GET/KVP
 ** or with POST/XML. Only SERVICE, VERSION (or WMTVER) and REQUEST are
@@ -100,12 +113,15 @@ static int msOWSPreParseRequest(cgiRequestObj *request,
     int i;
     /* parse KVP parameters service, version and request */
     for (i = 0; i < request->NumParams; ++i) {
-      if (EQUAL(request->ParamNames[i], "SERVICE")) {
+      if (ows_request->service == NULL &&
+          EQUAL(request->ParamNames[i], "SERVICE")) {
         ows_request->service = msStrdup(request->ParamValues[i]);
-      } else if (EQUAL(request->ParamNames[i], "VERSION")
-                 || EQUAL(request->ParamNames[i], "WMTVER")) { /* for WMS */
+      } else if (ows_request->version == NULL &&
+                 (EQUAL(request->ParamNames[i], "VERSION")
+                 || EQUAL(request->ParamNames[i], "WMTVER"))) { /* for WMS */
         ows_request->version = msStrdup(request->ParamValues[i]);
-      } else if (EQUAL(request->ParamNames[i], "REQUEST")) {
+      } else if (ows_request->request == NULL &&
+                 EQUAL(request->ParamNames[i], "REQUEST")) {
         ows_request->request = msStrdup(request->ParamValues[i]);
       }
 
@@ -117,6 +133,9 @@ static int msOWSPreParseRequest(cgiRequestObj *request,
   } else if (request->type == MS_POST_REQUEST) {
 #if defined(USE_LIBXML2)
     xmlNodePtr root = NULL;
+#if LIBXML_VERSION < 20900
+    xmlExternalEntityLoader oldExternalEntityLoader;
+#endif
 #elif defined(USE_GDAL)
     CPLXMLNode *temp;
 #endif
@@ -126,9 +145,24 @@ static int msOWSPreParseRequest(cgiRequestObj *request,
       return MS_FAILURE;
     }
 #if defined(USE_LIBXML2)
+#if LIBXML_VERSION < 20900
+    oldExternalEntityLoader = xmlGetExternalEntityLoader();
+    /* to avoid  XML External Entity vulnerability with libxml2 < 2.9 */
+    xmlSetExternalEntityLoader (dummyEntityLoader); 
+    bExternalEntityAsked = FALSE;
+#endif
     /* parse to DOM-Structure with libxml2 and get the root element */
     ows_request->document = xmlParseMemory(request->postrequest,
                                            strlen(request->postrequest));
+#if LIBXML_VERSION < 20900
+    xmlSetExternalEntityLoader (oldExternalEntityLoader); 
+    if( bExternalEntityAsked )
+    {
+        msSetError(MS_OWSERR, "XML parsing error: %s",
+                 "msOWSPreParseRequest()", "External entity fetch");
+        return MS_FAILURE;
+    }
+#endif
     if (ows_request->document == NULL
         || (root = xmlDocGetRootElement(ows_request->document)) == NULL) {
       xmlErrorPtr error = xmlGetLastError();
@@ -230,6 +264,14 @@ int msOWSDispatch(mapObj *map, cgiRequestObj *request, int ows_mode)
   }
 
   if (ows_request.service == NULL) {
+
+#ifdef USE_WFS_SVR
+    if( msOWSLookupMetadata(&(map->web.metadata), "FO", "cite_wfs2") != NULL ) {
+      status = msWFSException(map, "service", MS_OWS_ERROR_MISSING_PARAMETER_VALUE, NULL );
+    }
+    else
+#endif
+
     /* exit if service is not set */
     if(force_ows_mode) {
       msSetError( MS_MISCERR,
@@ -846,21 +888,19 @@ const char *msOWSLookupMetadataWithLanguage(hashTableObj *metadata,
     const char *namespaces, const char *name, const char* validated_language)
 {
   const char *value = NULL;
-  char *name2 = NULL;
-  size_t bufferSize = 0;
 
   if ( name && validated_language ) {
-    bufferSize = strlen(name)+strlen(validated_language)+2;
-    name2 = (char *) msSmallMalloc( bufferSize );
+    size_t bufferSize = strlen(name)+strlen(validated_language)+2;
+    char *name2 = (char *) msSmallMalloc( bufferSize );
     snprintf(name2, bufferSize, "%s.%s", name, validated_language);
     value = msOWSLookupMetadata(metadata, namespaces, name2);
+    free(name2);
   }
 
-  if (value == NULL) {
+  if ( name && !value ) {
     value = msOWSLookupMetadata(metadata, namespaces, name);
   }
 
-  msFree( name2 );
 
   return value;
 }
@@ -1016,7 +1056,7 @@ int msOWSMakeAllLayersUnique(mapObj *map)
 **
 */
 
-int msOWSNegotiateVersion(int requested_version, int supported_versions[], int num_supported_versions)
+int msOWSNegotiateVersion(int requested_version, const int supported_versions[], int num_supported_versions)
 {
   int i;
 
@@ -1157,6 +1197,26 @@ const char *msOWSGetSchemasLocation(mapObj *map)
   return schemas_location;
 }
 
+/* msOWSGetInspireSchemasLocation()
+**
+** schemas location is the root of the web tree where all Inspire-related
+** schemas can be found on this server.  These URLs must exist in order
+** to validate xml.
+**
+** Use value of "inspire_schemas_location" metadata
+*/
+const char *msOWSGetInspireSchemasLocation(mapObj *map)
+{
+  const char *schemas_location;
+
+  schemas_location = msLookupHashTable(&(map->web.metadata),
+                                       "inspire_schemas_location");
+  if (schemas_location == NULL)
+    schemas_location = "http://inspire.ec.europa.eu/schemas";
+
+  return schemas_location;
+}
+
 /* msOWSGetLanguage()
 **
 ** returns the language via MAP/WEB/METADATA/ows_language
@@ -1252,16 +1312,19 @@ char *msOWSGetLanguageFromList(mapObj *map, const char *namespaces, const char *
 ** Returns a status code; MS_NOERR if all ok, action_if_not_found otherwise
 */
 int msOWSPrintInspireCommonExtendedCapabilities(FILE *stream, mapObj *map, const char *namespaces,
-    int action_if_not_found, const char *tag_name,
-    const char *validated_language, const int service)
+    int action_if_not_found, const char *tag_name, const char* tag_ns,
+    const char *validated_language, const OWSServiceType service)
 {
 
   int metadataStatus = 0;
   int languageStatus = 0;
 
-  msIO_fprintf(stream, "  <%s>\n", tag_name);
+  if( tag_ns )
+    msIO_fprintf(stream, "  <%s %s>\n", tag_name, tag_ns);
+  else
+    msIO_fprintf(stream, "  <%s>\n", tag_name);
 
-  metadataStatus = msOWSPrintInspireCommonMetadata(stream, map, namespaces, action_if_not_found);
+  metadataStatus = msOWSPrintInspireCommonMetadata(stream, map, namespaces, action_if_not_found, service);
   languageStatus = msOWSPrintInspireCommonLanguages(stream, map, namespaces, action_if_not_found, validated_language);
 
   msIO_fprintf(stream, "  </%s>\n", tag_name);
@@ -1276,7 +1339,7 @@ int msOWSPrintInspireCommonExtendedCapabilities(FILE *stream, mapObj *map, const
 ** Returns a status code; MS_NOERR if all OK, action_if_not_found otherwise
 */
 int msOWSPrintInspireCommonMetadata(FILE *stream, mapObj *map, const char *namespaces,
-                                    int action_if_not_found)
+                                    int action_if_not_found, const OWSServiceType service)
 {
 
   int status = MS_NOERR;
@@ -1303,7 +1366,7 @@ int msOWSPrintInspireCommonMetadata(FILE *stream, mapObj *map, const char *names
       }
     }
   } else if (strcasecmp("embed",inspire_capabilities) == 0) {
-    msOWSPrintEncodeMetadata(stream, &(map->web.metadata), namespaces, "inspire_resourcelocator", OWS_NOERR, "    <inspire_common:ResourceLocator>\n      <inspire_common:URL>%s</inspire_common:URL>\n    </inspire_common:ResourceLocator>\n", NULL);
+    msOWSPrintEncodeMetadata(stream, &(map->web.metadata), namespaces, "inspire_resourcelocator", OWS_WARN, "    <inspire_common:ResourceLocator>\n      <inspire_common:URL>%s</inspire_common:URL>\n    </inspire_common:ResourceLocator>\n", NULL);
     msIO_fprintf(stream,"    <inspire_common:ResourceType>service</inspire_common:ResourceType>\n");
     msOWSPrintEncodeMetadata(stream, &(map->web.metadata), namespaces, "inspire_temporal_reference", OWS_WARN, "    <inspire_common:TemporalReference>\n      <inspire_common:DateOfLastRevision>%s</inspire_common:DateOfLastRevision>\n    </inspire_common:TemporalReference>\n", "");
     msIO_fprintf(stream, "    <inspire_common:Conformity>\n");
@@ -1318,8 +1381,11 @@ int msOWSPrintInspireCommonMetadata(FILE *stream, mapObj *map, const char *names
     msOWSPrintEncodeMetadata(stream, &(map->web.metadata), namespaces, "inspire_mpoc_email", OWS_WARN, "      <inspire_common:EmailAddress>%s</inspire_common:EmailAddress>\n", "");
     msIO_fprintf(stream, "    </inspire_common:MetadataPointOfContact>\n");
     msOWSPrintEncodeMetadata(stream, &(map->web.metadata), namespaces, "inspire_metadatadate", OWS_WARN, "      <inspire_common:MetadataDate>%s</inspire_common:MetadataDate>\n", "");
-    msIO_fprintf(stream,"    <inspire_common:SpatialDataServiceType>view</inspire_common:SpatialDataServiceType>\n");
-    msOWSPrintEncodeMetadata(stream, &(map->web.metadata), namespaces, "inspire_keyword", OWS_WARN, "    <inspire_common:MandatoryKeyword xsi:type='inspire_common:classificationOfSpatialDataService'>\n      <inspire_common:KeywordValue>%s</inspire_common:KeywordValue>\n    </inspire_common:MandatoryKeyword>\n", "");
+    if( service == OWS_WFS )
+        msIO_fprintf(stream,"    <inspire_common:SpatialDataServiceType>download</inspire_common:SpatialDataServiceType>\n");
+    else
+        msIO_fprintf(stream,"    <inspire_common:SpatialDataServiceType>view</inspire_common:SpatialDataServiceType>\n");
+    msOWSPrintEncodeMetadata(stream, &(map->web.metadata), namespaces, "inspire_keyword", OWS_WARN, "    <inspire_common:MandatoryKeyword>\n      <inspire_common:KeywordValue>%s</inspire_common:KeywordValue>\n    </inspire_common:MandatoryKeyword>\n", "");
   } else {
     status = action_if_not_found;
     if (OWS_WARN == action_if_not_found) {
@@ -1644,7 +1710,7 @@ int msOWSPrintURLType(FILE *stream, hashTableObj *metadata,
     value = msOWSLookupMetadata(metadata, namespaces, metadata_name);
     if(value != NULL) {
       encoded = msEncodeHTMLEntities(value);
-      buffer_size_tmp = strlen(type_format)+strlen(encoded);
+      buffer_size_tmp = strlen(type_format)+strlen(encoded)+1;
       type = (char*)malloc(buffer_size_tmp);
       snprintf(type, buffer_size_tmp, type_format, encoded);
       msFree(encoded);
@@ -1657,7 +1723,7 @@ int msOWSPrintURLType(FILE *stream, hashTableObj *metadata,
     value = msOWSLookupMetadata(metadata, namespaces, metadata_name);
     if(value != NULL) {
       encoded = msEncodeHTMLEntities(value);
-      buffer_size_tmp = strlen(width_format)+strlen(encoded);
+      buffer_size_tmp = strlen(width_format)+strlen(encoded)+1;
       width = (char*)malloc(buffer_size_tmp);
       snprintf(width, buffer_size_tmp, width_format, encoded);
       msFree(encoded);
@@ -1670,7 +1736,7 @@ int msOWSPrintURLType(FILE *stream, hashTableObj *metadata,
     value = msOWSLookupMetadata(metadata, namespaces, metadata_name);
     if(value != NULL) {
       encoded = msEncodeHTMLEntities(value);
-      buffer_size_tmp = strlen(height_format)+strlen(encoded);
+      buffer_size_tmp = strlen(height_format)+strlen(encoded)+1;
       height = (char*)malloc(buffer_size_tmp);
       snprintf(height, buffer_size_tmp, height_format, encoded);
       msFree(encoded);
@@ -1683,7 +1749,7 @@ int msOWSPrintURLType(FILE *stream, hashTableObj *metadata,
     value = msOWSLookupMetadata(metadata, namespaces, metadata_name);
     if(value != NULL) {
       encoded = msEncodeHTMLEntities(value);
-      buffer_size_tmp = strlen(urlfrmt_format)+strlen(encoded);
+      buffer_size_tmp = strlen(urlfrmt_format)+strlen(encoded)+1;
       urlfrmt = (char*)malloc(buffer_size_tmp);
       snprintf(urlfrmt, buffer_size_tmp, urlfrmt_format, encoded);
       msFree(encoded);
@@ -1696,7 +1762,7 @@ int msOWSPrintURLType(FILE *stream, hashTableObj *metadata,
     value = msOWSLookupMetadata(metadata, namespaces, metadata_name);
     if(value != NULL) {
       encoded = msEncodeHTMLEntities(value);
-      buffer_size_tmp = strlen(href_format)+strlen(encoded);
+      buffer_size_tmp = strlen(href_format)+strlen(encoded)+1;
       href = (char*)malloc(buffer_size_tmp);
       snprintf(href, buffer_size_tmp, href_format, encoded);
       msFree(encoded);
@@ -1868,8 +1934,8 @@ int msOWSPrintMetadataList(FILE *stream, hashTableObj *metadata,
         msIO_fprintf(stream, itemFormat, keywords[kw]);
       }
       if(endTag) msIO_fprintf(stream, "%s", endTag);
-      msFreeCharArray(keywords, numkeywords);
     }
+    msFreeCharArray(keywords, numkeywords);
     return MS_TRUE;
   }
   return MS_FALSE;
@@ -1919,8 +1985,8 @@ int msOWSPrintEncodeMetadataList(FILE *stream, hashTableObj *metadata,
         msFree(encoded);
       }
       if(endTag) msIO_fprintf(stream, "%s", endTag);
-      msFreeCharArray(keywords, numkeywords);
     }
+    msFreeCharArray(keywords, numkeywords);
     return MS_TRUE;
   }
   return MS_FALSE;
@@ -1961,8 +2027,8 @@ int msOWSPrintEncodeParamList(FILE *stream, const char *name,
       msFree(encoded);
     }
     if(endTag) msIO_fprintf(stream, "%s", endTag);
-    msFreeCharArray(items, numitems);
   }
+  msFreeCharArray(items, numitems);
 
   return status;
 }
@@ -2023,7 +2089,7 @@ void msOWSPrintEX_GeographicBoundingBox(FILE *stream, const char *tabspace,
 */
 void msOWSPrintLatLonBoundingBox(FILE *stream, const char *tabspace,
                                  rectObj *extent, projectionObj *srcproj,
-                                 projectionObj *wfsproj, int nService)
+                                 projectionObj *wfsproj, OWSServiceType nService)
 {
   const char *pszTag = "LatLonBoundingBox";  /* The default for WMS */
   rectObj ext;
@@ -2091,24 +2157,24 @@ void msOWSPrintBoundingBox(FILE *stream, const char *tabspace,
 
   for( i = 0; i < num_epsgs; i++) {
     value = epsgs[i];
-    memcpy(&ext, extent, sizeof(rectObj));
+    if( value && *value) {
+      memcpy(&ext, extent, sizeof(rectObj));
 
-    /* reproject the extents for each SRS's bounding box */
-    msInitProjection(&proj);
-    if (msLoadProjectionStringEPSG(&proj, (char *)value) == 0) {
-      if (msProjectionsDiffer(srcproj, &proj) == MS_TRUE) {
-        msProjectRect(srcproj, &proj, &ext);
+      /* reproject the extents for each SRS's bounding box */
+      msInitProjection(&proj);
+      if (msLoadProjectionStringEPSG(&proj, (char *)value) == 0) {
+        if (msProjectionsDiffer(srcproj, &proj) == MS_TRUE) {
+          msProjectRect(srcproj, &proj, &ext);
+        }
+        /*for wms 1.3.0 we need to make sure that we present the BBOX with
+          a reversed axes for some espg codes*/
+        if (wms_version >= OWS_1_3_0 && value && strncasecmp(value, "EPSG:", 5) == 0) {
+          msAxisNormalizePoints( &proj, 1, &(ext.minx), &(ext.miny) );
+          msAxisNormalizePoints( &proj, 1, &(ext.maxx), &(ext.maxy) );
+        }
       }
-      /*for wms 1.3.0 we need to make sure that we present the BBOX with
-        a reversed axes for some espg codes*/
-      if (wms_version >= OWS_1_3_0 && value && strncasecmp(value, "EPSG:", 5) == 0) {
-        msAxisNormalizePoints( &proj, 1, &(ext.minx), &(ext.miny) );
-        msAxisNormalizePoints( &proj, 1, &(ext.maxx), &(ext.maxy) );
-      }
-    }
-    msFreeProjection( &proj );
+      msFreeProjection( &proj );
 
-    if( value != NULL ) {
       encoded = msEncodeHTMLEntities(value);
       if (wms_version >= OWS_1_3_0)
         msIO_fprintf(stream, "%s<BoundingBox CRS=\"%s\"\n"
@@ -2315,6 +2381,11 @@ void msOWSProcessException(layerObj *lp, const char *pszFname,
 
     fseek(fp, 0, SEEK_END);
     nBufSize = ftell(fp);
+    if(nBufSize < 0) {
+      msSetError(MS_IOERR, NULL, "msOWSProcessException()");
+      fclose(fp);
+      return;
+    }
     rewind(fp);
     pszBuf = (char*)malloc((nBufSize+1)*sizeof(char));
     if (pszBuf == NULL) {
@@ -2770,10 +2841,11 @@ outputFormatObj* msOwsIsOutputFormatValid(mapObj *map, const char *format,
         if (mimetype && strcasecmp(mimetype, format) == 0)
           break;
       }
-      msFreeCharArray(tokens, n);
       if (i < n)
         psFormat = msSelectOutputFormat( map, format);
     }
+    if(tokens)
+      msFreeCharArray(tokens, n);
   }
 
   return psFormat;
