@@ -29,38 +29,7 @@
 
 #include "mapserver.h"
 #include "mapcopy.h"
-
-int computeLabelStyle(labelStyleObj *s, labelObj *l, fontSetObj *fontset,
-                      double scalefactor, double resolutionfactor)
-{
-  INIT_LABEL_STYLE(*s);
-  if(!MS_VALID_COLOR(l->color))
-    return MS_FAILURE;
-  if(l->size == -1)
-    return MS_FAILURE;
-
-  s->size = l->size;
-  if(l->type == MS_TRUETYPE) {
-    s->size *=  scalefactor;
-    s->size = MS_MAX(s->size, l->minsize*resolutionfactor);
-    s->size = MS_MIN(s->size, l->maxsize*resolutionfactor);
-    if (!fontset) {
-      msSetError(MS_TTFERR, "No fontset defined.","computeLabelStyle()");
-      return (MS_FAILURE);
-    }
-
-    if (!l->font || !(*l->font)) {
-      return (MS_FAILURE);
-    }
-
-    if(MS_FAILURE == msFontsetLookupFonts(l->font,&s->numfonts,fontset,s->fonts)) {
-      return MS_FAILURE;
-    }
-  }
-  s->rotation = l->angle * MS_DEG_TO_RAD;
-  s->antialias = l->antialias;
-  return MS_SUCCESS;
-}
+#include "fontcache.h"
 
 void computeSymbolStyle(symbolStyleObj *s, styleObj *src, symbolObj *symbol, double scalefactor,
     double resolutionfactor)
@@ -141,25 +110,16 @@ int preloadSymbol(symbolSetObj *symbolset, symbolObj *symbol, rendererVTableObj
   case MS_SYMBOL_VECTOR:
   case MS_SYMBOL_ELLIPSE:
   case MS_SYMBOL_SIMPLE:
+  case (MS_SYMBOL_TRUETYPE):
     break;
   case (MS_SYMBOL_SVG):
 #if defined(USE_SVG_CAIRO) || defined(USE_RSVG)
-      return msPreloadSVGSymbol(symbol);
+    return msPreloadSVGSymbol(symbol);
 #else
-      msSetError(MS_SYMERR, "SVG symbol support is not enabled.", "preloadSymbol()");
-      return MS_FAILURE;
+    msSetError(MS_SYMERR, "SVG symbol support is not enabled.", "preloadSymbol()");
+    return MS_FAILURE;
 #endif
-      break;
-  case (MS_SYMBOL_TRUETYPE): {
-    if(!symbol->full_font_path)
-      symbol->full_font_path =  msStrdup(msLookupHashTable(&(symbolset->fontset->fonts),
-          symbol->font));
-    if(!symbol->full_font_path) {
-      msSetError(MS_MEMERR,"allocation error", "preloadSymbol()");
-      return MS_FAILURE;
-    }
-  }
-  break;
+    break;
   case (MS_SYMBOL_PIXMAP): {
     if(!symbol->pixmap_buffer) {
       if(MS_SUCCESS != msPreloadImageSymbol(renderer,symbol))
@@ -221,10 +181,47 @@ tileCacheObj *addTileCache(imageObj *img,
   return(cachep);
 }
 
+/* helper function to center glyph on the desired point */
+int WARN_UNUSED drawGlyphMarker(imageObj *img, face_element *face, glyph_element *glyphc, double px, double py, int size, double rotation,
+    colorObj *clr, colorObj *oclr, int olwidth)
+{
+  double ox, oy;
+  textPathObj path;
+  glyphObj glyph;
+  rendererVTableObj *renderer = img->format->vtable;
+  if(!renderer->renderGlyphs) return MS_FAILURE;
+  path.numglyphs = 1;
+  glyph.glyph = glyphc;
+  glyph.face = face;
+  path.glyphs = &glyph;
+  path.glyph_size = size;
+  glyph.rot = rotation;
+  ox = (glyphc->metrics.maxx + glyphc->metrics.minx) / 2.0;
+  oy = (glyphc->metrics.maxy + glyphc->metrics.miny) / 2.0;
+  if(rotation != 0) {
+    double sina, cosa;
+    double rox,roy;
+    sina = sin(rotation);
+    cosa = cos(rotation);
+    rox = ox * cosa - oy * sina;
+    roy = ox * sina + oy * cosa;
+    px -= rox;
+    py += roy;
+    glyph.pnt.x = px;
+    glyph.pnt.y = py;
+  } else {
+    glyph.pnt.x = px - ox;
+    glyph.pnt.y = py + oy;
+  }
+  return renderer->renderGlyphs(img, &path, clr, oclr, olwidth);
+}
+
+
 imageObj *getTile(imageObj *img, symbolObj *symbol,  symbolStyleObj *s, int width, int height,
                   int seamlessmode)
 {
   tileCacheObj *tile;
+  int status = MS_SUCCESS;
   rendererVTableObj *renderer = img->format->vtable;
   if(width==-1 || height == -1) {
     width=height=MS_MAX(symbol->sizex,symbol->sizey);
@@ -235,42 +232,63 @@ imageObj *getTile(imageObj *img, symbolObj *symbol,  symbolStyleObj *s, int widt
     imageObj *tileimg;
     double p_x,p_y;
     tileimg = msImageCreate(width,height,img->format,NULL,NULL,img->resolution, img->resolution, NULL);
+    if(UNLIKELY(!tileimg)) {
+      return NULL;
+    }
     if(!seamlessmode) {
       p_x = width/2.0;
       p_y = height/2.0;
       switch(symbol->type) {
         case (MS_SYMBOL_TRUETYPE):
-          renderer->renderTruetypeSymbol(tileimg, p_x, p_y, symbol, s);
+        {
+          unsigned int unicode;
+          glyph_element *glyphc;
+          face_element *face = msGetFontFace(symbol->font, &img->map->fontset);
+          if(UNLIKELY(!face)) { status = MS_FAILURE; break; }
+          msUTF8ToUniChar(symbol->character, &unicode);
+          unicode = msGetGlyphIndex(face,unicode);
+          glyphc = msGetGlyphByIndex(face, MS_MAX(MS_NINT(s->scale),1), unicode);
+          if(UNLIKELY(!glyphc)) { status = MS_FAILURE; break; }
+          status = drawGlyphMarker(tileimg, face, glyphc, p_x, p_y, s->scale, s->rotation,
+                s->color, s->outlinecolor, s->outlinewidth);
+        }
           break;
         case (MS_SYMBOL_PIXMAP):
-          renderer->renderPixmapSymbol(tileimg, p_x, p_y, symbol, s);
+          status = msPreloadImageSymbol(renderer,symbol);
+          if(UNLIKELY(status == MS_FAILURE)) { break; }
+          status = renderer->renderPixmapSymbol(tileimg, p_x, p_y, symbol, s);
           break;
         case (MS_SYMBOL_ELLIPSE):
-          renderer->renderEllipseSymbol(tileimg, p_x, p_y,symbol, s);
+          status = renderer->renderEllipseSymbol(tileimg, p_x, p_y,symbol, s);
           break;
         case (MS_SYMBOL_VECTOR):
-          renderer->renderVectorSymbol(tileimg, p_x, p_y, symbol, s);
+          status = renderer->renderVectorSymbol(tileimg, p_x, p_y, symbol, s);
           break;
 
         case (MS_SYMBOL_SVG):
 #if defined(USE_SVG_CAIRO) || defined(USE_RSVG)
-          if (renderer->supports_svg) {
-            if(renderer->renderSVGSymbol(tileimg, p_x, p_y, symbol, s) != MS_SUCCESS) {
-              return NULL;
-            }
-          } else {
-            if (msRenderRasterizedSVGSymbol(tileimg,p_x,p_y,symbol, s) != MS_SUCCESS) {
-              return NULL;
+          status = msPreloadSVGSymbol(symbol);
+          if(LIKELY(status == MS_SUCCESS)) {
+            if (renderer->supports_svg) {
+              status = renderer->renderSVGSymbol(tileimg, p_x, p_y, symbol, s);
+            } else {
+              status = msRenderRasterizedSVGSymbol(tileimg,p_x,p_y,symbol, s);
             }
           }
 #else
           msSetError(MS_SYMERR, "SVG symbol support is not enabled.", "getTile()");
-          return NULL;
+          status = MS_FAILURE;
 #endif
           break;
         default:
+          msSetError(MS_SYMERR, "Unknown symbol type %d", "getTile()", symbol->type);
+          status = MS_FAILURE;
           break;
       }
+      if(UNLIKELY(status == MS_FAILURE)) {
+        msFreeImage(tileimg);
+        return NULL;
+      }
     } else {
       /*
        * in seamless mode, we render the the symbol 9 times on a 3x3 grid to account for
@@ -285,23 +303,61 @@ imageObj *getTile(imageObj *img, symbolObj *symbol,  symbolStyleObj *s, int widt
         for(j=1; j<=3; j++) {
           p_y = (j+0.5) * height;
           switch(symbol->type) {
+            case (MS_SYMBOL_TRUETYPE):
+            {
+              unsigned int unicode;
+              glyph_element *glyphc;
+              face_element *face = msGetFontFace(symbol->font, &img->map->fontset);
+              if(UNLIKELY(!face)) { status = MS_FAILURE; break; }
+              msUTF8ToUniChar(symbol->character, &unicode);
+              unicode = msGetGlyphIndex(face,unicode);
+              glyphc = msGetGlyphByIndex(face, MS_MAX(MS_NINT(s->scale),1), unicode);
+              if(UNLIKELY(!glyphc)) { status = MS_FAILURE; break; }
+              status = drawGlyphMarker(tileimg, face, glyphc, p_x, p_y, s->scale, s->rotation,
+                    s->color, s->outlinecolor, s->outlinewidth);
+            }
+              break;
+            case (MS_SYMBOL_PIXMAP):
+              status = msPreloadImageSymbol(renderer,symbol);
+              if(UNLIKELY(status == MS_FAILURE)) { break; }
+              status = renderer->renderPixmapSymbol(tile3img, p_x, p_y, symbol, s);
+              break;
+            case (MS_SYMBOL_ELLIPSE):
+              status = renderer->renderEllipseSymbol(tile3img, p_x, p_y,symbol, s);
+              break;
             case (MS_SYMBOL_VECTOR):
-              renderer->renderVectorSymbol(tile3img, p_x, p_y, symbol, s);
+              status = renderer->renderVectorSymbol(tile3img, p_x, p_y, symbol, s);
               break;
             default:
               msSetError(MS_SYMERR, "BUG: Seamless mode is only for vector symbols", "getTile()");
               return NULL;
           }
+          if(UNLIKELY(status == MS_FAILURE)) {
+            msFreeImage(tile3img);
+            return NULL;
+          }
         }
       }
+      if(UNLIKELY(status == MS_FAILURE)) {
+        msFreeImage(tile3img);
+        return NULL;
+      }
 
-      MS_IMAGE_RENDERER(tile3img)->getRasterBufferHandle(tile3img,&tmpraster);
-      renderer->mergeRasterBuffer(tileimg,
+      status = MS_IMAGE_RENDERER(tile3img)->getRasterBufferHandle(tile3img,&tmpraster);
+      if(UNLIKELY(status == MS_FAILURE)) {
+        msFreeImage(tile3img);
+        return NULL;
+      }
+      status = renderer->mergeRasterBuffer(tileimg,
                                   &tmpraster,
                                   1.0,width,height,0,0,width,height
                                  );
       msFreeImage(tile3img);
     }
+    if(UNLIKELY(status == MS_FAILURE)) {
+      msFreeImage(tileimg);
+      return NULL;
+    }
     tile = addTileCache(img,tileimg,symbol,s,width,height);
   }
   return tile->image;
@@ -316,17 +372,22 @@ int msImagePolylineMarkers(imageObj *image, shapeObj *p, symbolObj *symbol,
   pointObj point;
   double original_rotation = style->rotation;
   double symbol_width,symbol_height;
-  int ret = MS_FAILURE;
+  glyph_element *glyphc = NULL;
+  face_element *face;
+  int ret = MS_SUCCESS;
   if(symbol->type != MS_SYMBOL_TRUETYPE) {
     symbol_width = MS_MAX(1,symbol->sizex*style->scale);
     symbol_height = MS_MAX(1,symbol->sizey*style->scale);
   } else {
-    rectObj rect;
-    if(MS_SUCCESS != renderer->getTruetypeTextBBox(renderer,&symbol->full_font_path,1,style->scale,
-        symbol->character,&rect,NULL,0))
-      return MS_FAILURE;
-    symbol_width=rect.maxx-rect.minx;
-    symbol_height=rect.maxy-rect.miny;
+    unsigned int unicode;
+    msUTF8ToUniChar(symbol->character, &unicode);
+    face = msGetFontFace(symbol->font, &image->map->fontset);
+    if(UNLIKELY(!face)) return MS_FAILURE;
+    unicode = msGetGlyphIndex(face,unicode);
+    glyphc = msGetGlyphByIndex(face, MS_MAX(MS_NINT(style->scale),1), unicode);
+    if(UNLIKELY(!glyphc)) return MS_FAILURE;
+    symbol_width = glyphc->metrics.maxx - glyphc->metrics.minx;
+    symbol_height = glyphc->metrics.maxy - glyphc->metrics.miny;
   }
   for(i=0; i<p->numlines; i++) {
     int line_in = 0;
@@ -394,7 +455,8 @@ int msImagePolylineMarkers(imageObj *image, shapeObj *p, symbolObj *symbol,
             ret = renderer->renderVectorSymbol(image, point.x, point.y, symbol, style);
             break;
           case MS_SYMBOL_TRUETYPE:
-            ret = renderer->renderTruetypeSymbol(image, point.x, point.y, symbol, style);
+            ret = drawGlyphMarker(image, face, glyphc, point.x, point.y, style->scale, style->rotation,
+                style->color, style->outlinecolor, style->outlinewidth);
             break;
           case (MS_SYMBOL_SVG):
 #if defined(USE_SVG_CAIRO) || defined(USE_RSVG)
@@ -464,7 +526,8 @@ int msImagePolylineMarkers(imageObj *image, shapeObj *p, symbolObj *symbol,
               ret = renderer->renderVectorSymbol(image, point.x, point.y, symbol, style);
               break;
             case MS_SYMBOL_TRUETYPE:
-              ret = renderer->renderTruetypeSymbol(image, point.x, point.y, symbol, style);
+              ret = drawGlyphMarker(image, face, glyphc, point.x, point.y, style->scale, style->rotation,
+                  style->color, style->outlinecolor, style->outlinewidth);
               break;
             case (MS_SYMBOL_SVG):
 #if defined(USE_SVG_CAIRO) || defined(USE_RSVG)
@@ -489,9 +552,10 @@ int msImagePolylineMarkers(imageObj *image, shapeObj *p, symbolObj *symbol,
   return ret;
 }
 
-int msDrawLineSymbol(symbolSetObj *symbolset, imageObj *image, shapeObj *p,
+int msDrawLineSymbol(mapObj *map, imageObj *image, shapeObj *p,
                      styleObj *style, double scalefactor)
 {
+  int status = MS_SUCCESS;
   if (image) {
     if (MS_RENDERER_PLUGIN(image->format)) {
       rendererVTableObj *renderer = image->format->vtable;
@@ -504,10 +568,10 @@ int msDrawLineSymbol(symbolSetObj *symbolset, imageObj *image, shapeObj *p,
       if (p->numlines == 0)
         return MS_SUCCESS;
 
-      if (style->symbol >= symbolset->numsymbols || style->symbol < 0)
+      if (style->symbol >= map->symbolset.numsymbols || style->symbol < 0)
         return MS_SUCCESS; /* no such symbol, 0 is OK   */
 
-      symbol = symbolset->symbol[style->symbol];
+      symbol = map->symbolset.symbol[style->symbol];
       /* store a reference to the renderer to be used for freeing */
       symbol->renderer = renderer;
 
@@ -546,13 +610,15 @@ int msDrawLineSymbol(symbolSetObj *symbolset, imageObj *image, shapeObj *p,
         else {
           /* msSetError(MS_MISCERR,"no color defined for line styling","msDrawLineSymbol()");
            * not really an error */
-          return MS_SUCCESS;
+          status = MS_SUCCESS;
+          goto line_cleanup;
         }
-        renderer->renderLine(image,offsetLine,&s);
+        status = renderer->renderLine(image,offsetLine,&s);
       } else {
         symbolStyleObj s;
-        if(preloadSymbol(symbolset, symbol, renderer) != MS_SUCCESS) {
-          return MS_FAILURE;
+        if(preloadSymbol(&map->symbolset, symbol, renderer) != MS_SUCCESS) {
+          status = MS_FAILURE;
+          goto line_cleanup;
         }
 
         INIT_SYMBOL_STYLE(s);
@@ -562,10 +628,10 @@ int msDrawLineSymbol(symbolSetObj *symbolset, imageObj *image, shapeObj *p,
         /* compute a markerStyle and use it on the line */
         if(style->gap<0) {
           /* special function that treats any other symbol used as a marker, not a brush */
-          msImagePolylineMarkers(image,offsetLine,symbol,&s,-s.gap,
+          status = msImagePolylineMarkers(image,offsetLine,symbol,&s,-s.gap,
                                  style->initialgap * finalscalefactor, 1);
         } else if(style->gap>0) {
-          msImagePolylineMarkers(image,offsetLine,symbol,&s,s.gap,
+          status = msImagePolylineMarkers(image,offsetLine,symbol,&s,s.gap,
                                  style->initialgap * finalscalefactor,0);
         } else {
           if(renderer->renderLineTiled != NULL) {
@@ -581,40 +647,44 @@ int msDrawLineSymbol(symbolSetObj *symbolset, imageObj *image, shapeObj *p,
             if(pw<1) pw=1;
             if(ph<1) ph=1;
             tile = getTile(image, symbol,&s,pw,ph,0);
-            renderer->renderLineTiled(image, offsetLine, tile);
+            status = renderer->renderLineTiled(image, offsetLine, tile);
           } else {
             msSetError(MS_RENDERERERR, "renderer does not support brushed lines", "msDrawLineSymbol()");
-            return MS_FAILURE;
+            status = MS_FAILURE;
           }
         }
       }
 
+line_cleanup:
       if(offsetLine!=p) {
         msFreeShape(offsetLine);
         msFree(offsetLine);
       }
+      return status;
     } else if( MS_RENDERER_IMAGEMAP(image->format) )
-      msDrawLineSymbolIM(symbolset, image, p, style, scalefactor);
+      msDrawLineSymbolIM(map, image, p, style, scalefactor);
     else {
       msSetError(MS_RENDERERERR, "unsupported renderer", "msDrawShadeSymbol()");
-      return MS_FAILURE;
+      status = MS_FAILURE;
     }
   } else {
-    return MS_FAILURE;
+    status = MS_FAILURE;
   }
-  return MS_SUCCESS;
+  return status;
 }
 
-int msDrawShadeSymbol(symbolSetObj *symbolset, imageObj *image, shapeObj *p, styleObj *style, double scalefactor)
+int msDrawShadeSymbol(mapObj *map, imageObj *image, shapeObj *p, styleObj *style, double scalefactor)
 {
   int ret = MS_SUCCESS;
+  symbolObj *symbol;
   if (!p)
     return MS_SUCCESS;
   if (p->numlines <= 0)
     return MS_SUCCESS;
 
-  if (style->symbol >= symbolset->numsymbols || style->symbol < 0)
+  if (style->symbol >= map->symbolset.numsymbols || style->symbol < 0)
     return MS_SUCCESS; /* no such symbol, 0 is OK */
+  symbol = map->symbolset.symbol[style->symbol];
 
   /*
    * if only an outlinecolor was defined, and not a color,
@@ -623,10 +693,10 @@ int msDrawShadeSymbol(symbolSetObj *symbolset, imageObj *image, shapeObj *p, sty
    * this behavior is kind of a mapfile hack, and must be
    * kept for backwards compatibility
    */
-  if (symbolset->symbol[style->symbol]->type != MS_SYMBOL_PIXMAP && symbolset->symbol[style->symbol]->type != MS_SYMBOL_SVG ) {
+  if (symbol->type != MS_SYMBOL_PIXMAP && symbol->type != MS_SYMBOL_SVG ) {
     if (!MS_VALID_COLOR(style->color)) {
       if(MS_VALID_COLOR(style->outlinecolor))
-        return msDrawLineSymbol(symbolset, image, p, style, scalefactor);
+        return msDrawLineSymbol(map, image, p, style, scalefactor);
       else {
         /* just do nothing if no color has been set */
         return MS_SUCCESS;
@@ -637,7 +707,6 @@ int msDrawShadeSymbol(symbolSetObj *symbolset, imageObj *image, shapeObj *p, sty
     if (MS_RENDERER_PLUGIN(image->format)) {
       rendererVTableObj *renderer = image->format->vtable;
       shapeObj *offsetPolygon = NULL;
-      symbolObj *symbol = symbolset->symbol[style->symbol];
       /* store a reference to the renderer to be used for freeing */
       if(style->symbol)
         symbol->renderer = renderer;
@@ -698,8 +767,9 @@ int msDrawShadeSymbol(symbolSetObj *symbolset, imageObj *image, shapeObj *p, sty
         imageObj *tile;
         int seamless = 0;
 
-        if(preloadSymbol(symbolset,symbol,renderer) != MS_SUCCESS) {
-          return MS_FAILURE;
+        if(preloadSymbol(&map->symbolset,symbol,renderer) != MS_SUCCESS) {
+          ret = MS_FAILURE;
+          goto cleanup;
         }
 
         INIT_SYMBOL_STYLE(s);
@@ -756,18 +826,18 @@ cleanup:
       }
       return ret;
     } else if( MS_RENDERER_IMAGEMAP(image->format) )
-      msDrawShadeSymbolIM(symbolset, image, p, style, scalefactor);
+      msDrawShadeSymbolIM(map, image, p, style, scalefactor);
   }
   return ret;
 }
 
-int msDrawMarkerSymbol(symbolSetObj *symbolset,imageObj *image, pointObj *p, styleObj *style,
+int msDrawMarkerSymbol(mapObj *map, imageObj *image, pointObj *p, styleObj *style,
                        double scalefactor)
 {
   int ret = MS_SUCCESS;
   if (!p)
     return MS_SUCCESS;
-  if (style->symbol >= symbolset->numsymbols || style->symbol <= 0)
+  if (style->symbol >= map->symbolset.numsymbols || style->symbol <= 0)
     return MS_SUCCESS; /* no such symbol, 0 is OK   */
 
   if (image) {
@@ -775,10 +845,10 @@ int msDrawMarkerSymbol(symbolSetObj *symbolset,imageObj *image, pointObj *p, sty
       rendererVTableObj *renderer = image->format->vtable;
       symbolStyleObj s;
       double p_x,p_y;
-      symbolObj *symbol = symbolset->symbol[style->symbol];
+      symbolObj *symbol = map->symbolset.symbol[style->symbol];
       /* store a reference to the renderer to be used for freeing */
       symbol->renderer = renderer;
-      if(preloadSymbol(symbolset,symbol,renderer) != MS_SUCCESS) {
+      if(preloadSymbol(&map->symbolset,symbol,renderer) != MS_SUCCESS) {
         return MS_FAILURE;
       }
 
@@ -815,7 +885,9 @@ int msDrawMarkerSymbol(symbolSetObj *symbolset,imageObj *image, pointObj *p, sty
       if(symbol->anchorpoint_x != 0.5 || symbol->anchorpoint_y != 0.5) {
         double sx,sy;
         double ox, oy;
-        msGetMarkerSize(symbolset, style, &sx, &sy, scalefactor);
+        if(UNLIKELY(MS_FAILURE == msGetMarkerSize(map, style, &sx, &sy, scalefactor))) {
+          return MS_FAILURE;
+        }
         ox = (0.5 - symbol->anchorpoint_x) * sx;
         oy = (0.5 - symbol->anchorpoint_y) * sy;
         if(s.rotation != 0) {
@@ -844,9 +916,15 @@ int msDrawMarkerSymbol(symbolSetObj *symbolset,imageObj *image, pointObj *p, sty
       }
       switch (symbol->type) {
         case (MS_SYMBOL_TRUETYPE): {
-          assert(symbol->full_font_path);
-          ret = renderer->renderTruetypeSymbol(image, p_x, p_y, symbol, &s);
-
+          unsigned int unicode;
+          glyph_element *glyphc;
+          face_element *face = msGetFontFace(symbol->font, &map->fontset);
+          if(UNLIKELY(!face)) return MS_FAILURE;
+          msUTF8ToUniChar(symbol->character,&unicode);
+          unicode = msGetGlyphIndex(face,unicode);
+          glyphc = msGetGlyphByIndex(face, MS_MAX(MS_NINT(s.scale),1), unicode);
+          if(UNLIKELY(!glyphc)) return MS_FAILURE;
+          ret = drawGlyphMarker(image, face, glyphc, p_x, p_y, s.scale, s.rotation, s.color, s.outlinecolor, s.outlinewidth);
         }
         break;
         case (MS_SYMBOL_PIXMAP): {
@@ -880,164 +958,110 @@ int msDrawMarkerSymbol(symbolSetObj *symbolset,imageObj *image, pointObj *p, sty
       }
       return ret;
     } else if( MS_RENDERER_IMAGEMAP(image->format) )
-      msDrawMarkerSymbolIM(symbolset, image, p, style, scalefactor);
+      msDrawMarkerSymbolIM(map, image, p, style, scalefactor);
 
   }
   return ret;
 }
 
 
-
-
-
-/*
-** Render the text (no background effects) for a label.
- */
-int msDrawText(imageObj *image, pointObj labelPnt, char *string,
-               labelObj *label, fontSetObj *fontset, double scalefactor)
+int msDrawLabelBounds(mapObj *map, imageObj *image, label_bounds *bnds, styleObj *style, double scalefactor)
 {
-  int nReturnVal = -1;
-  if (image) {
-    if (MS_RENDERER_PLUGIN(image->format)) {
-      labelStyleObj s;
-      rendererVTableObj *renderer = image->format->vtable;
-      double x, y;
-      if (!string || !strlen(string))
-        return (0); /* not errors, just don't want to do anything */
-
-
-      if(computeLabelStyle(&s,label,fontset,scalefactor,image->resolutionfactor) == MS_FAILURE) {
-        return MS_FAILURE;
-      }
-      if(s.rotation == 0 && !MS_RENDERER_KML(image->format)) {
-        x = MS_NINT(labelPnt.x);
-        y = MS_NINT(labelPnt.y);
-      } else {
-        x = labelPnt.x;
-        y = labelPnt.y;
-      }
-      if (label->type == MS_TRUETYPE) {
-        if(MS_VALID_COLOR(label->shadowcolor)) {
-          s.color = &label->shadowcolor;
-          /* FIXME labelpoint for rotated label */
-          renderer->renderGlyphs(image,
-                                 x + scalefactor * label->shadowsizex,y + scalefactor * label->shadowsizey,
-                                 &s,string);
-        }
-
-        s.color= &label->color;
-        if(MS_VALID_COLOR(label->outlinecolor)) {
-          s.outlinecolor = &label->outlinecolor;
-          s.outlinewidth = label->outlinewidth * s.size/label->size;
-        }
-        return renderer->renderGlyphs(image,x,y,&s,string);
-      } else if(label->type == MS_BITMAP) {
-        s.size = MS_NINT(s.size);
-        s.color= &label->color;
-        s.size = MS_MIN(s.size,5); /* only have 5 bitmap fonts */
-        if(!renderer->supports_bitmap_fonts || !renderer->bitmapFontMetrics[MS_NINT(s.size)]) {
-          msSetError(MS_RENDERERERR, "selected renderer does not support bitmap fonts or this particular size", "msDrawText()");
-          return MS_FAILURE;
-        }
-        return renderer->renderBitmapGlyphs(image,x,y,&s,string);
-      }
-    } else if( MS_RENDERER_IMAGEMAP(image->format) )
-      nReturnVal = msDrawTextIM(image, labelPnt, string, label, fontset, scalefactor);
+  /*
+   * helper function to draw label bounds, where we might have only a rectObj and not
+   * a lineObj/shapeObj
+   */
+  shapeObj shape;
+  shape.numlines = 1;
+  if(bnds->poly) {
+    shape.line = bnds->poly;
+    return msDrawShadeSymbol(map,image,&shape,style,scalefactor);
+  } else {
+    pointObj pnts1[5];
+    lineObj l;
+    l.point = pnts1;
+    l.numpoints = 5;
+    pnts1[0].x = pnts1[1].x = pnts1[4].x = bnds->bbox.minx;
+    pnts1[2].x = pnts1[3].x = bnds->bbox.maxx;
+    pnts1[0].y = pnts1[3].y = pnts1[4].y = bnds->bbox.miny;
+    pnts1[1].y = pnts1[2].y = bnds->bbox.maxy;
+    shape.line = &l; // must return from this block
+    return msDrawShadeSymbol(map,image,&shape,style,scalefactor);
   }
-  return nReturnVal;
 }
 
-int msDrawTextLine(imageObj *image, char *string, labelObj *label, labelPathObj *labelpath, fontSetObj *fontset, double scalefactor)
+int msDrawTextSymbol(mapObj *map, imageObj *image, pointObj labelPnt, textSymbolObj *ts)
 {
-  int nReturnVal = MS_SUCCESS;
-  if(image) {
-    if (MS_RENDERER_PLUGIN(image->format)) {
+  rendererVTableObj *renderer = image->format->vtable;
+  colorObj *c = NULL, *oc = NULL;
+  int ow;
+  assert(ts->textpath);
 
-      rendererVTableObj *renderer = image->format->vtable;
-      labelStyleObj s;
-      if (!string || !strlen(string))
-        return (MS_SUCCESS); /* not errors, just don't want to do anything */
-      if(computeLabelStyle(&s, label, fontset, scalefactor,image->resolutionfactor) != MS_SUCCESS) return MS_FAILURE;
-      if (label->type == MS_TRUETYPE) {
-        if(renderer->renderGlyphsLine) {
-          if(MS_VALID_COLOR(label->outlinecolor)) {
-            s.outlinecolor = &(label->outlinecolor);
-            s.outlinewidth = s.size/label->size * label->outlinewidth;
-          } else {
-            s.outlinewidth = 0;
-            s.outlinecolor = NULL;
-          }
-          s.color = &(label->color);
-          nReturnVal = renderer->renderGlyphsLine(image,labelpath,&s,string);
-        } else {
-          /*
-           * if the renderer doesn't support direct labelPath usage,
-           * decompose the string and send it down character by character
-           */
-          const char* string_ptr = string;
-          int i;
-          double x, y;
-          char glyph[11];
-
-          /*
-           * we first render all the outlines if present, so that the
-           * joining of characters stays correct
-           */
-          if(MS_VALID_COLOR(label->outlinecolor)) {
-            s.outlinecolor = &(label->outlinecolor);
-            s.outlinewidth = s.size/label->size * label->outlinewidth;
-            for (i = 0; i < labelpath->path.numpoints; i++) {
-              if (msGetNextGlyph(&string_ptr, glyph) == -1)
-                break; /* Premature end of string??? */
-              s.rotation = labelpath->angles[i];
-              x = labelpath->path.point[i].x;
-              y = labelpath->path.point[i].y;
-              nReturnVal = renderer->renderGlyphs(image, x, y, &s, glyph);
-              if(nReturnVal != MS_SUCCESS) {
-                return nReturnVal;
-              }
-            }
-            string_ptr = string; /* reset to beginning of string */
-          }
-          s.outlinecolor = NULL;
-          s.outlinewidth = 0;
-          s.color = &(label->color);
-          for (i = 0; i < labelpath->path.numpoints; i++) {
-            if (msGetNextGlyph(&string_ptr, glyph) == -1)
-              break; /* Premature end of string??? */
-
-            s.rotation = labelpath->angles[i];
-            x = labelpath->path.point[i].x;
-            y = labelpath->path.point[i].y;
-
-            nReturnVal = renderer->renderGlyphs(image, x, y, &s, glyph);
-            if(nReturnVal != MS_SUCCESS) {
-              return nReturnVal;
-            }
-          }
-        }
+  if(!ts->textpath->absolute) {
+    int g;
+    double cosa,sina;
+    double x = labelPnt.x;
+    double y = labelPnt.y;
+    if(ts->rotation != 0) {
+      cosa = cos(ts->rotation);
+      sina = sin(ts->rotation);
+      for(g=0;g<ts->textpath->numglyphs;g++) {
+        double ox = ts->textpath->glyphs[g].pnt.x;
+        double oy = ts->textpath->glyphs[g].pnt.y;
+        ts->textpath->glyphs[g].pnt.x = ox * cosa + oy * sina + x;
+        ts->textpath->glyphs[g].pnt.y = -ox * sina + oy * cosa + y;
+        ts->textpath->glyphs[g].rot = ts->rotation;
+      }
+    } else {
+      for(g=0;g<ts->textpath->numglyphs;g++) {
+        ts->textpath->glyphs[g].pnt.x += x;
+        ts->textpath->glyphs[g].pnt.y += y;
       }
     }
   }
-
-  return nReturnVal;
+  if(MS_VALID_COLOR(ts->label->color))
+    c = &ts->label->color;
+  if(MS_VALID_COLOR(ts->label->outlinecolor))
+    oc = &ts->label->outlinecolor;
+  ow = MS_NINT((double)ts->label->outlinewidth * ((double)ts->textpath->glyph_size / (double)ts->label->size));
+  if(!renderer->renderGlyphs) return MS_FAILURE;
+  return renderer->renderGlyphs(image,ts->textpath,c,oc,ow);
+  
 }
 
-int msCircleDrawShadeSymbol(symbolSetObj *symbolset, imageObj *image, pointObj *p, double r, styleObj *style, double scalefactor)
+/************************************************************************/
+/*                          msCircleDrawLineSymbol                      */
+/*                                                                      */
+/************************************************************************/
+int msCircleDrawLineSymbol(mapObj *map, imageObj *image, pointObj *p, double r, styleObj *style, double scalefactor)
 {
   shapeObj *circle;
+  int status;
   if (!image) return MS_FAILURE;
   circle = msRasterizeArc(p->x, p->y, r, 0, 360, 0);
   if (!circle) return MS_FAILURE;
-  msDrawShadeSymbol(symbolset, image, circle, style, scalefactor);
+  status = msDrawLineSymbol(map, image, circle, style, scalefactor);
   msFreeShape(circle);
   msFree(circle);
-  return MS_SUCCESS;
+  return status;
 }
 
-int msDrawPieSlice(symbolSetObj *symbolset, imageObj *image, pointObj *p, styleObj *style, double radius, double start, double end)
+int msCircleDrawShadeSymbol(mapObj *map, imageObj *image, pointObj *p, double r, styleObj *style, double scalefactor)
 {
+  shapeObj *circle;
+  int status;
+  if (!image) return MS_FAILURE;
+  circle = msRasterizeArc(p->x, p->y, r, 0, 360, 0);
+  if (!circle) return MS_FAILURE;
+  status = msDrawShadeSymbol(map, image, circle, style, scalefactor);
+  msFreeShape(circle);
+  msFree(circle);
+  return status;
+}
 
+int msDrawPieSlice(mapObj *map, imageObj *image, pointObj *p, styleObj *style, double radius, double start, double end)
+{
+  int status;
   shapeObj *circle;
   double center_x = p->x;
   double center_y = p->y;
@@ -1048,8 +1072,59 @@ int msDrawPieSlice(symbolSetObj *symbolset, imageObj *image, pointObj *p, styleO
   }
   circle = msRasterizeArc(center_x, center_y, radius, start, end, 1);
   if (!circle) return MS_FAILURE;
-  msDrawShadeSymbol(symbolset, image, circle, style, 1.0);
+  status = msDrawShadeSymbol(map, image, circle, style, 1.0);
   msFreeShape(circle);
   msFree(circle);
-  return MS_SUCCESS;
+  return status;
+}
+
+/*
+ * RFC 49 implementation
+ * if an outlinewidth is used:
+ *  - augment the style's width to account for the outline width
+ *  - swap the style color and outlinecolor
+ *  - draw the shape (the outline) in the first pass of the
+ *    caching mechanism
+ */
+
+void msOutlineRenderingPrepareStyle(styleObj *pStyle, mapObj *map, layerObj *layer, imageObj *image)
+{
+  colorObj tmp;
+
+  if (pStyle->outlinewidth > 0) {
+    /* adapt width (must take scalefactor into account) */
+    pStyle->width += (pStyle->outlinewidth / (layer->scalefactor/image->resolutionfactor)) * 2;
+    pStyle->minwidth += pStyle->outlinewidth * 2;
+    pStyle->maxwidth += pStyle->outlinewidth * 2;
+    pStyle->size += (pStyle->outlinewidth/layer->scalefactor*(map->resolution/map->defresolution));
+
+    /*swap color and outlinecolor*/
+    tmp = pStyle->color;
+    pStyle->color = pStyle->outlinecolor;
+    pStyle->outlinecolor = tmp;
+  }
+}
+
+/*
+ * RFC 49 implementation: switch back the styleobj to its
+ * original state, so the line fill will be drawn in the
+ * second pass of the caching mechanism
+ */
+
+void msOutlineRenderingRestoreStyle(styleObj *pStyle, mapObj *map, layerObj *layer, imageObj *image)
+{
+  colorObj tmp;
+
+  if (pStyle->outlinewidth > 0) {
+    /* reset widths to original state */
+    pStyle->width -= (pStyle->outlinewidth / (layer->scalefactor/image->resolutionfactor)) * 2;
+    pStyle->minwidth -= pStyle->outlinewidth * 2;
+    pStyle->maxwidth -= pStyle->outlinewidth * 2;
+    pStyle->size -= (pStyle->outlinewidth/layer->scalefactor*(map->resolution/map->defresolution));
+
+    /*reswap colors to original state*/
+    tmp = pStyle->color;
+    pStyle->color = pStyle->outlinecolor;
+    pStyle->outlinecolor = tmp;
+  }
 }
