@@ -416,8 +416,9 @@ static char *msBuildWFSLayerGetURL(mapObj *map, layerObj *lp, rectObj *bbox,
   /*      mutually exclusive.                                             */
   /* -------------------------------------------------------------------- */
   if (psParams->pszFilter) {
-    snprintf(pszURL + strlen(pszURL), bufferSize-strlen(pszURL), "&FILTER=%s",
-             msEncodeUrl(psParams->pszFilter));
+    char *encoded_filter = msEncodeUrl(psParams->pszFilter);
+    snprintf(pszURL + strlen(pszURL), bufferSize-strlen(pszURL), "&FILTER=%s",encoded_filter);
+    free(encoded_filter);
   } else {
 	  /*
 	   * take care about the axis order for WFS 1.1
@@ -654,8 +655,12 @@ int msPrepareWFSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
   pasReqInfo[(*numRequests)].debug = lp->debug;
 
   if (msHTTPAuthProxySetup(&(map->web.metadata), &(lp->metadata),
-                           pasReqInfo, *numRequests, map, "FO") != MS_SUCCESS)
+                           pasReqInfo, *numRequests, map, "FO") != MS_SUCCESS) {
+    if (psParams) {
+      msWFSFreeParamsObj(psParams);
+    }
     return MS_FAILURE;
+  }
   
   /* ------------------------------------------------------------------
    * Pre-Open the layer now, (i.e. alloc and fill msWFSLayerInfo inside
@@ -684,7 +689,6 @@ int msPrepareWFSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
 
   if (psParams) {
     msWFSFreeParamsObj(psParams);
-    psParams = NULL;
   }
   return nStatus;
 
